% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/empiricalquantileregression.R
\name{QR.2d}
\alias{QR.2d}
\title{Use rolling windows quantiles or additive quantile regression to calculate local quantiles of R|W}
\usage{
QR.2d(
  r,
  w,
  method = c("empirical", "smooth"),
  tau = 0.95,
  J = 60,
  wqg = 0.08,
  quantilespacing = TRUE,
  k.smooth = 15,
  bs.smooth = "tp"
)
}
\arguments{
\item{r}{Values of R}

\item{w}{Values of W}

\item{method}{Character string: "empirical" for empirical rolling-windows quantiles, "smooth" for generalized additive quantile regression using the R library qgam}

\item{tau}{value in (0,1) - find the tau-quantile}

\item{J}{number of windows for rolling window estimate}

\item{wqg}{Gap between the quantiles / values of w to calculate the sizes of the rolling windows}

\item{quantilespacing}{logical. If TRUE then size of rolling windows is determined by the quantiles of W. If FALSE, rolling windows are of a fixed length equal to wqg}

\item{k.smooth}{Number of basis functions to use if method="smooth"}

\item{bs.smooth}{Type of basis functions to use if method="smooth" (See qgam library for further details).}
}
\value{
List with three elements (i) "wpts",
 containing w values: for method="empirical", these are the midpoints of the w values of each rolling window; for method="smooth", these are values on a grid.
 (ii) "r.tau.wpts" giving an estimate of the tau-quantile for R|W corresponding to each of the w values in wpts,
 and (iii) "r0w" containing a threshold value of R for each w in the data.
}
\description{
Use rolling windows quantiles or additive quantile regression to calculate local quantiles of R|W
}
\author{
Jenny Wadsworth
}
